import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.*;
import java.util.ArrayList;
import javax.swing.*;

public class SlotCarUI {
	public SlotCarUI(String a, int g1, int g2){
		pathLoc = a;
		graphData1Disp = g1;
		graphData2Disp = g2;
	}
	public SlotCarUI(String a, int g1, int g2, ParticleGenerator code, int ds){
		pathLoc = a;
		graphData1Disp = g1;
		graphData2Disp = g2;
		dataLoc = code;
		switch(ds){
		case 1:dataset = "471data/inner/";imgName = "imu";break;
		case 2:dataset = "471data/outer/";imgName = "imu";break;
		case 3:dataset = "471data/both_again/";imgName = "both_again";break;
		}
	}
	
	String dataset = "";
	String imgName = "";
	ParticleGenerator dataLoc;
	
	ArrayList<Vector2D> points = new ArrayList<Vector2D>();
	
	public static final int WINDOWLENGTH = 940;
	public static final int WINDOWWIDTH = 920;
	public static final int GRAPHHEIGHT = 190;
	public static final int GRAPHWIDTH = 900;

	public void run(){
		setup();
		start();
	}

	int graphData1Disp;
	int graphData2Disp;
	JLabel image;
	String pathLoc;
	JTextArea text;
	Graph graph1;
	Graph graph2;
	PandSpace key;
	JPanel trackOverlay;
	
	public void setup(){
		dispSetup();
		loadInTrackPieces();
		constructTrack();
	}

	public void start(){
		getData();
	}


	
	
	public void getData(){
		findAndSetMinMax();
		String dataPath = "imu_sg2_data";
		try {
			BufferedReader in = new BufferedReader(new FileReader(pathLoc + dataset + dataPath));
	        String str = in.readLine();
	        int lastTimestamp = 0;
	        graph1.graphLoc.setText(getDiscOfGraph(graphData1Disp));
	        graph2.graphLoc.setText(getDiscOfGraph(graphData2Disp));
	        while (str != null) {
	        	//mainPane.repaint();
	        	graphZeroLine(datasetMax[graphData1Disp], datasetMin[graphData1Disp], graph1);
		        graphZeroLine(datasetMax[graphData2Disp], datasetMin[graphData2Disp], graph2);
		        
	        	String[] dataFrag = str.split(" ");
	        	ArrayList<String> data = new ArrayList<String>();
	        	for(int i = 0; i < dataFrag.length; i++){
	        		if(!dataFrag[i].equalsIgnoreCase("")){
	        			data.add(dataFrag[i]);
	        		}
	        	}
	        	long startRenderTime = System.currentTimeMillis();//get the starting render time
	        	updateDisplay(data,graphData1Disp,graphData2Disp);
	        	if(dataLoc != null){
	        		if(dataLoc.hasTrackData()){
	        			dataLoc.setTrackData( trackData );
	        		}
	        		points = dataLoc.getParticles(stringDataToIntData(data));
	        	}
	        	mainPane.repaint();
	        	long totRenderTime = System.currentTimeMillis() - startRenderTime;//calculate the total time it took to render
	        	int currentTimestamp = new Integer(data.get(2)).intValue();
	        	int totFrameTime = (currentTimestamp - lastTimestamp);//calculate how long the frame took in real time
	        	int totSleepTime = totFrameTime - (int)totRenderTime;//calculate the difference, aka, how long to sleep
	        	try {
	        		if(totSleepTime > 0){
	        			Thread.sleep(totSleepTime);
	        		}
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
	        	lastTimestamp = currentTimestamp;
	        	while(key.pause && !key.space){
		        	try {Thread.sleep(10);} catch (InterruptedException e) {}
	        	}
	        	
	        	//getmax(data);
	        	str = in.readLine();
	        }
	        in.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		/*System.out.println("{ " + max_frame_id + ", " +max_server_timestamp + ", " + max_client_timestamp+ ", " + max_car_image_pixel_location
				+ ", " + max_roll+ ", " + max_pitch+ ", " +max_yaw + ", " + max_car_angle+ ", " + max_filtered_postion
				+ ", " + max_filtered_velocity+ " }");*/
		//disp();
	}
	
	public void findAndSetMinMax(){
		double maxs[] = {-10000,-10000,-10000,-10000,-10000,-10000,-10000,-10000,-10000,-10000,-10000,-10000,-10000,-10000,-10000};
		double mins[] = {10000,10000,10000,10000,10000,10000,10000,10000,10000,10000,10000,10000,10000,10000,10000};
		String dataPath = "imu_sg2_data";
		BufferedReader in;
		try {
			in = new BufferedReader(new FileReader(pathLoc + dataset + dataPath));
		    String str = in.readLine();
		    while (str != null) {
		    	String[] dataFrag = str.split(" ");
		    	ArrayList<String> datas = new ArrayList<String>();
		    	for(int i = 0; i < dataFrag.length; i++){
		    		if(!dataFrag[i].equalsIgnoreCase("")){
		    			datas.add(dataFrag[i]);
		    		}
		    	}
		    	ArrayList<Double> data = stringDataToIntData(datas);
		    	for(int i = 0; i < data.size(); i++){
		    		if(data.get(i) > maxs[i]){
		    			maxs[i] = data.get(i);
		    		}
		    		if(data.get(i) < mins[i]){
		    			mins[i] = data.get(i);
		    		}
		    	}
		    	//System.out.println(data.get(0));
		    	str = in.readLine();
		    }
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		datasetMax = maxs;
		datasetMin = mins;
    }
	
	
	
	
	
	
	
	public ArrayList<Double> stringDataToIntData(ArrayList<String> a){
		ArrayList<Double> tr = new ArrayList<Double>();
		for(String b:a){
			tr.add(new Double(b).doubleValue());
		}
		return tr;
	}
	
	
	
	
	public TrackPiece bankedLeft;
	public TrackPiece bankedRight;
	public TrackPiece left;
	public TrackPiece right;
	public TrackPiece smallLeft;
	public TrackPiece smallRight;
	public TrackPiece smallStraight;
	public TrackPiece straight;
	
	public void loadInTrackPieces(){
		bankedLeft = loadInTrackPiece("data/sections/bankedleft");
		bankedRight = loadInTrackPiece("data/sections/bankedright");
		left = loadInTrackPiece("data/sections/left");
		right = loadInTrackPiece("data/sections/right");
		smallLeft = loadInTrackPiece("data/sections/smallleft");
		smallRight = loadInTrackPiece("data/sections/smallright");
		smallStraight = loadInTrackPiece("data/sections/smallstraight");
		straight = loadInTrackPiece("data/sections/straight");
	}
	
	public TrackPiece loadInTrackPiece(String dataPath){
		BufferedReader in;
		TrackPiece p = null;
		try {
			in = new BufferedReader(new FileReader(pathLoc + dataPath));
	        String str = in.readLine();
	        String data = "";
	        while (str != null) {
	        	data += str.trim() + ";";
	        	str = in.readLine();
	        }
	        data = data.replaceAll(" ", "");
	        data = data.replaceAll("[a-zA-Z]","");
	        String[] rData = data.split(";");
	        p = new TrackPiece(new Double(rData[0]).intValue(), new Double(rData[1]).doubleValue(), new Double(rData[2]).doubleValue(), new Double(rData[3]).doubleValue());
		} catch (IOException e) {e.printStackTrace();
		}
		return p;
	}
	
	public String[] loadInTrackData(){
		String dataPath = "data/track";
		BufferedReader in;
		try {
			in = new BufferedReader(new FileReader(pathLoc + dataPath));

	        String str = in.readLine();
	        String data = "";
	        while (str != null) {
	        	data += str.trim() + "-";
	        	str = in.readLine();
	        }
	        data = data.replaceAll("--", "-");
	        return data.split("-");
		
		} catch (IOException e) {e.printStackTrace();
		}
		return null;
	}
	
	public TrackData trackData;
	public Transform2D transform;	
	public void constructTrack()	{
		trackData = new TrackData();
		//transform = new Transform2D(593.559959, 189.542434, 1.644992, 1.797643, 1);
		transform = new Transform2D(482.22, 46.88, 0.071, 1.78, 1);
		
		String[] trackSpec = loadInTrackData();
		double dist = 0;
		for(int i = 0; i < trackSpec.length; i++){
			TrackPiece nextPiece;
			if(trackSpec[i].equalsIgnoreCase("bankedleft")){
				nextPiece = bankedLeft;
			}else if(trackSpec[i].equalsIgnoreCase("bankedright")){
				nextPiece = bankedRight;
			}else if(trackSpec[i].equalsIgnoreCase("left")){
				nextPiece = left;
			}else if(trackSpec[i].equalsIgnoreCase("right")){
				nextPiece = right;
			}else if(trackSpec[i].equalsIgnoreCase("smallleft")){
				nextPiece = smallLeft;
			}else if(trackSpec[i].equalsIgnoreCase("smallright")){
				nextPiece = smallRight;
			}else if(trackSpec[i].equalsIgnoreCase("smallstraight")){
				nextPiece = smallStraight;
			}else /*if(trackSpec[i].equalsIgnoreCase("straight"))*/{
				nextPiece = straight;
			}			
			dist = trackData.addSegment(new TrackSegment(nextPiece, dist));
		}
		
		trackData.updateTrack();		
	}
	
	public String getDiscOfGraph(int i){
		return getDiscOfNum(i)+ " (vs time)";
	}
	public String getDiscOfNum(int i){
		switch(i){
		case 0: return "FrameId";
		case 1: return "SrvTime";
		case 2: return "ClientTime";
		case 3: return "PixX";
		case 4: return "PixY";
		case 5: return "UnfilteredLoc";
		case 6: return "Roll";
		case 7: return "Pitch";
		case 8: return "Yaw";
		case 9: return "AngleFromIMU";
		case 10: return "FilteredLoc";
		case 11: return "FilteredVel";
		case 12: return "AngleFromVisionAndModel";
		case 13: return "sin(angle_vis)";
		case 14: return "cos(angle_vis)";
		}
		return "";
	}
	
	
	public static double[] datasetMax = { 882.0, 4.36756258E8, 64829.0, 1365.24, 0.54, 0.25, 3.12, 3.12, 1365.12, 316.51, 122.36, 7.97 };
	public static double[] datasetMin = { 0.0, 0.0, 18.0, -1.0, -0.18, -0.25, -3.13, -3.13, 0.67, 5.07, -83.28, -323.19 };
	
	public static double getTrackLength(){
		return datasetMax[5];
	}
	
	public void updateDisplay(ArrayList<String> data, int graph1dataset, int graph2dataset){
		updateImage(new Integer(data.get(0)).intValue());
		text.setText(getText(data));
		graphPoints(data, graph1dataset, datasetMax[graph1dataset], datasetMin[graph1dataset], graph1);
		graphPoints(data, graph2dataset, datasetMax[graph2dataset], datasetMin[graph2dataset], graph2);
	}

	public void graphPoints(ArrayList<String> data, int i, double maxValOfI, double minValOfI, Graph graph){
		double currentX = new Double(data.get(2)).doubleValue();
		double currentY = new Double(data.get(i)).doubleValue();
		//64829
		//int startX =  (int)(graph.lastx/64829*GRAPHWIDTH);
		//int startY =  GRAPHHEIGHT - (int)((graph.lasty-minValOfI)/(maxValOfI-minValOfI)*GRAPHHEIGHT);
		int endX =    (int)(currentX/64829*GRAPHWIDTH);
		int endY =    GRAPHHEIGHT - (int)((currentY-minValOfI)/(maxValOfI-minValOfI)*GRAPHHEIGHT);
		
		//graph.graph.drawLine(startX, startY, endX, endY);
		graph.graphPointsX.add(new Integer(endX));
		graph.graphPointsY.add(new Integer(endY));
		
		graph.lastx = currentX;
		graph.lasty = currentY;
	}
	
	public void graphZeroLine(double maxValOfI, double minValOfI, Graph graph){
	}
	
	public String getText(ArrayList<String> data){
		String a = "";
		for(int i = 0; i < data.size(); i++){
			a += getDiscOfNum(i) + " " + data.get(i) + "\n";
		}
		return a;
	}
	
	public void updateImage(int imageNum){
		image.setIcon(new ImageIcon(pathLoc + dataset + "/" + imgName + imageNum + ".jpeg"));
	}
	
	JLayeredPane mainPane;
	public void dispSetup(){
		ImageIcon a = new ImageIcon(pathLoc + dataset + "/" + imgName + 0 + ".jpeg");
		JFrame frame = new JFrame();
		frame.setBounds(40,40,WINDOWWIDTH,WINDOWLENGTH);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setResizable(false);
		frame.setBackground(Color.black);
		
		mainPane = new JLayeredPane();
		mainPane.setBounds(0,0,WINDOWWIDTH,WINDOWLENGTH);
		
		image = new JLabel(a);
		image.setBounds(0,0,a.getIconWidth(),a.getIconHeight());
		//graphLoc = new JLabel();
		//graphLoc.setBounds(300, a.getIconHeight(), 300, 300);
		//graphLoc.setBackground(Color.black);
		
		JTextArea graphLoc1 = new JTextArea();
		graphLoc1.setBounds(2, a.getIconHeight() + 2, 200, 20);
		graphLoc1.setEditable(false);
		graphLoc1.setBackground(Color.white);
		
		JTextArea graphLoc2 = new JTextArea();
		graphLoc2.setBounds(2, a.getIconHeight()+ GRAPHHEIGHT + 4 + 20 + 2, 200, 20);
		graphLoc2.setEditable(false);
		graphLoc2.setBackground(Color.white);
		
		JTextArea data = new JTextArea();
		data.setBounds(a.getIconWidth(),250,200,40);
		data.setEditable(false);
		data.setText("hit p to pause/unpause \n" +
				"hit space when paused to step");
		
		
		text = new JTextArea();
		text.setBounds(a.getIconWidth(),0,300,240);
		text.setEditable(false);
		
		key = new PandSpace();
		mainPane.addKeyListener(key);
		
		//trackOverlay = new TrackOverlay();
		trackOverlay = new Track();
		trackOverlay.setBounds(0,0,a.getIconWidth(),a.getIconHeight());
		trackOverlay.setOpaque(false);
		

		graph1 = new Graph(graphLoc1, graphLoc1.getGraphics(),datasetMax[graphData1Disp], datasetMin[graphData1Disp]);
		graph1.setBounds(2, a.getIconHeight() + 4 + 20, GRAPHWIDTH, GRAPHHEIGHT);
		graph1.setEditable(false);
		graph1.setOpaque(true);
		graph1.setBackground(Color.white);
		
		graph2 = new Graph(graphLoc2, graphLoc2.getGraphics(),datasetMax[graphData1Disp], datasetMin[graphData1Disp]);
		graph2.setBounds(2, a.getIconHeight()+ GRAPHHEIGHT + 8 + 40, GRAPHWIDTH, GRAPHHEIGHT);
		graph2.setEditable(false);
		graph2.setOpaque(true);
		graph2.setBackground(Color.white);
		
		mainPane.add(graph1, new Integer(1));
		mainPane.add(graph2, new Integer(1));
		mainPane.add(image , new Integer(1));
		mainPane.add(text , new Integer(1));
		mainPane.add(data, new Integer(1));
		mainPane.add(graphLoc1, new Integer(2));
		mainPane.add(graphLoc2, new Integer(2));
		mainPane.add(trackOverlay, new Integer(2));
		
		frame.setContentPane(mainPane);
		frame.setVisible(true);
		mainPane.requestFocus();
		
		/*for(int i = 0; i < 100; i++){
			image.setIcon(new ImageIcon(pathLoc + "image/imu" + i + ".jpeg"));
		}*/
		frame.repaint();
	}

	public static ArrayList<Color> particleColors = null;
	
	public static void setParticleColors(ArrayList<Color> a){
		particleColors = a;
	}
	
	@SuppressWarnings("serial")
	public class Track extends JPanel{
		public Track(){
		}
		public void paint(Graphics g){
			if(trackData != null && !trackData.isEmpty())	{
				Vector2D lastPoint = transform.toImage(trackData.segment(0).getStartPosition(0.5));
				int lastPointx = (int)(lastPoint.x);
				int lastPointy = (int)(lastPoint.y);
				g.setColor(Color.DARK_GRAY);
				for(double d = 0.0; d < trackData.length; d += 10.0){
					Vector2D currPoint = transform.toImage(trackData.pointAtDistance(d, 0.5));
					int pointx = (int)(currPoint.x); 
					int pointy = (int)(currPoint.y); 
					g.drawLine(pointx, pointy, lastPointx, lastPointy);
					//make it thicker
					//g.drawLine(pointx+1, pointy+1, lastPointx+1, lastPointy+1);
					lastPointx = pointx;
					lastPointy = pointy;
				}

				lastPoint = transform.toImage(trackData.segment(0).getStartPosition(0));
				lastPointx = (int)(lastPoint.x);
				lastPointy = (int)(lastPoint.y);
				g.setColor(Color.red);
				for(double d = 0.0; d < trackData.length; d += 10.0){
					Vector2D currPoint = transform.toImage(trackData.pointAtDistance(d, 0));
					int pointx = (int)(currPoint.x); 
					int pointy = (int)(currPoint.y); 
					g.drawLine(pointx, pointy, lastPointx, lastPointy);											
					//make it thicker
					//g.drawLine(pointx+1, pointy+1, lastPointx+1, lastPointy+1);
					lastPointx = pointx;
					lastPointy = pointy;
				}
				
				lastPoint = transform.toImage(trackData.segment(0).getStartPosition(1));
				lastPointx = (int)(lastPoint.x);
				lastPointy = (int)(lastPoint.y);
				g.setColor(Color.green);
				for(double d = 0.0; d < trackData.length; d += 10.0){
					Vector2D currPoint = transform.toImage(trackData.pointAtDistance(d, 1));
					int pointx = (int)(currPoint.x); 
					int pointy = (int)(currPoint.y); 
					g.drawLine(pointx, pointy, lastPointx, lastPointy);											
					//make it thicker
					//g.drawLine(pointx+1, pointy+1, lastPointx+1, lastPointy+1);
					lastPointx = pointx;
					lastPointy = pointy;
				}						
			}
			if(points != null){
				for(int i = 0; i < points.size(); i++){
					Vector2D a = points.get(i);
					Vector2D currPoint = transform.toImage(a);
					
					/*if(a.y < 0){
						currPoint = transform.toImage(a);
					}
					*/
					//Vector2D currPoint = transform.toImage(trackData.pointAtDistance(a.x,0.5));
					if(particleColors != null && (i < particleColors.size())){
						g.setColor(particleColors.get(i));
					}else{
						g.setColor(Color.orange);
					}
					g.drawRect((int)currPoint.x-3, (int)currPoint.y-3, 6, 6);
					//g.drawLine((int)currPoint.x, (int)currPoint.y, (int)currPoint.x, (int)currPoint.y);
					//g.drawRect((int)currPoint.x, (int)5, 6, 6);
				}
			}
		}
	}
		
	
	@SuppressWarnings("serial")
	public class Graph extends JTextArea{
		JTextArea graphLoc;
		Graphics graph;
		double lastx;
		double lasty;
		ArrayList<Integer> graphPointsX;
		ArrayList<Integer> graphPointsY;
		double minValOfI;
		double maxValOfI;
		
		public Graph(JTextArea gl, Graphics g, double min, double max){
			graphLoc = gl;
			graph = g;
			lastx = 0;
			lasty = 0;
			minValOfI = min;
			maxValOfI = max;
			graphPointsX = new ArrayList<Integer>();
			graphPointsY = new ArrayList<Integer>();
		}

		public void paint(Graphics g)
		{
			g.setColor(Color.white);
			g.fillRect(-1, -1, GRAPHWIDTH+1, GRAPHHEIGHT+1);
			g.setColor(Color.black);
			int y =  GRAPHHEIGHT - (int)((0-minValOfI)/(maxValOfI-minValOfI)*GRAPHHEIGHT);
			g.drawLine(0, y, GRAPHWIDTH, y);
			for(int i = 0; i < graphPointsX.size()-1; i++){
				g.drawLine(graphPointsX.get(i).intValue(), graphPointsY.get(i).intValue(), graphPointsX.get(i+1).intValue(), graphPointsY.get(i+1).intValue());
			}
		}
	}

	public class PandSpace implements KeyListener{
		boolean space = false;
		boolean p = false;
		boolean pause = false;
		public void keyPressed(KeyEvent e) {
			switch(e.getKeyCode()){
			case KeyEvent.VK_SPACE: space = true;break;
			case KeyEvent.VK_P: p = true;
				pause = !pause;
				break;
			}
		}
		public void keyReleased(KeyEvent e) {
			switch(e.getKeyCode()){
			case KeyEvent.VK_SPACE: space = false;break;
			case KeyEvent.VK_P: p = false;break;
			}
		}
		public void keyTyped(KeyEvent e) {
		}
	}
	
}
	










