import java.util.ArrayList;

public class TrackData {
	ArrayList<TrackSegment> track; 
	double length;
	
	public TrackData()	{
		track = new ArrayList<TrackSegment>();
		length = 0; 
	}

	// construct the track after created...
	public void updateTrack()	{
		int size = numSegments(); 
		if(size <= 0)
			return; 
		track.get(0).setStartPosition(new Vector2D(0,0));
		track.get(0).setStartDirection(new Vector2D(1,0));		
		for(int idx = 1; idx < size; idx++)	{
			track.get(idx).setStartPosition(track.get(idx-1).getEndPosition(0.5));
			track.get(idx).setStartDirection(track.get(idx-1).getEndDirection());
		}
	}
	
	// important: used for particle evaluation...
	public double distance(double estimate, Vector2D observation, double track)
	{
		Vector2D est2D = pointAtDistance(estimate, track); 
		return (est2D.subtract(observation)).length();
	}

	// important: used for particle evaluation...
	// note the delta_angle returned is between -PI ~ PI...
	public double angle(double init_pos, double delta_dist, double track)
	{
		Vector2D init_pose = directionAtDistance(init_pos, track);
		Vector2D curr_pose = directionAtDistance(init_pos + delta_dist, track);
		double delta = Math.atan2(init_pose.y, init_pose.x) - Math.atan2(curr_pose.y, curr_pose.x);
		if (delta > Math.PI)
			delta -= 2.0 * Math.PI;
		if (delta < -Math.PI)
			delta += 2.0 * Math.PI;
		return delta;
	}
		
	// helper functions......
	public double addSegment(TrackSegment ts)	{
		length = ts.getEndDist();
		track.add(ts);
		return length;
	}
	
	public Vector2D pointAtDistance(double d, double track )	{
		TrackSegment s = segmentAtDistance(d);
		return s.pointAtDistance(d-s.startDist, track);
	}
	
	public Vector2D directionAtDistance(double d, double track )	{
		TrackSegment s = segmentAtDistance(d);
		return s.directionAtDistance(d-s.startDist, track);
	}
		
	public boolean isEmpty()	{
		return (track.size() <= 0);
	}
	
	public int numSegments()	{
		return track.size();
	}
	
	public TrackSegment segment(int idx)	{
		idx = idx % numSegments(); 
		return track.get(idx);
	}
	
	public int segmentIDAtDistance(double d){
		if(d > length || d < 0)
			d -= (int)(d/length) * length; 
		
		int size = numSegments();
		for(int idx = 0; idx < size; idx++)	{
			if(track.get(idx).getEndDist() > d && track.get(idx).getStartDist() <= d)
				return idx; 
		}
		
		return 0; 
	}
	
	public TrackSegment segmentAtDistance(double d){
		return segment(segmentIDAtDistance(d));
	}	
}
