import java.lang.Math; 

public class TrackSegment {
	double length; 
	double width; 
	double curvature;

	double startDist;
	
	Vector2D startPos; 
	Vector2D startDir; 
	Vector2D endDir; 
	Vector2D endPos;
	
	public TrackSegment(TrackPiece p, double start1d)	{
		length = p.length;
		width = p.width;
		curvature = p.curvature;
		
		startDist = start1d;
		
		startPos = new Vector2D();
		startDir = new Vector2D();
		endPos = new Vector2D();
		endDir = new Vector2D();
	}

	public Vector2D calcEndDirection(double a)	{
		double so = Math.sin(a);
		double co = Math.cos(a);
		return new Vector2D(co*startDir.x - so*startDir.y, so*startDir.x + co*startDir.y);
	}
		
	public Vector2D calcEndPosition(double a, double l, double track)	{
		Vector2D dir = startDir.add(calcEndDirection(a));
		double factor = l / (dir.length() + 0.000001); 
		if(a != 0)
			factor *= (2 * Math.sin(a/2) / a); 
		
		Vector2D offset = calcEndDirection(a).orthogonal().multCon((0.5-track)*width);		
		return startPos.add(dir.multCon(factor).add(offset));
	}
	
	public Vector2D getStartPosition(double track) {
		Vector2D offset = startDir.orthogonal().multCon((0.5-track)*width);
		return startPos.add(offset);
	}
	
	public Vector2D getStartDirection() {
		return startDir;
	}
	
	public Vector2D getEndPosition(double track) {
		Vector2D offset = endDir.orthogonal().multCon((0.5-track)*width);
		return endPos.add(offset);
	}
	
	public Vector2D getEndDirection() {
		return endDir;
	}
	
	public double getStartDist()	{
		return startDist; 
	}
	
	public double getEndDist()	{
		return (startDist+length); 
	}
	
	public void setStartPosition(Vector2D sp)	{
		startPos = sp;
		endPos = calcEndPosition(curvature/180.0*3.1415927, length, 0.5); 		
	}
	
	public void setStartDirection(Vector2D sd)	{
		startDir = sd.normalize();
		endDir = calcEndDirection(curvature/180.0*3.1415927);
		endPos = calcEndPosition(curvature/180.0*3.1415927, length, 0.5); 		
	}
	
	public Vector2D pointAtDistance(double d, double track)	{
		if(curvature == 0)
			return getStartPosition(track).add(startDir.multCon(d));
		else
			return calcEndPosition(d/length*curvature/180.0*3.1415927, d, track);
	}
	
	public Vector2D directionAtDistance(double d, double track ) {
		return calcEndDirection(d/length*curvature/180.0*3.1415927);
	}
}
